from functools import wraps

from .exceptions import MsbAuthExceptions


def require_permissions(*permission_classes, _any: bool = False):
	def decorator_function(_decorated_function):
		@wraps(_decorated_function)
		def wrapper_function(cls, request, *args, **kwargs):
			evaluator = any if _any else all

			if not evaluator([permission().has_permission(request, cls) for permission in permission_classes]):
				raise MsbAuthExceptions.UnauthorizedAccess

			return _decorated_function(cls, *args, **dict(request=request, **kwargs))

		return wrapper_function

	return decorator_function



from rest_framework.permissions import BasePermission

from .exceptions import MsbAuthExceptions
from .users import TokenUser


class Permissions:
	class Authenticated(BasePermission):

		def has_permission(self, request, view) -> bool:
			return bool(request.user and request.user.is_authenticated)

		def has_object_permission(self, request, view, obj):
			return True

		def check(self, request, view, obj=None) -> bool:
			if not self.has_permission(request, view, ):
				raise MsbAuthExceptions.UnauthorizedAccess
			return True

	class Admin(Authenticated):
		def has_permission(self, request, view) -> bool:
			return bool(
				super().has_permission(request, view) and
				request.user.is_admin
			)

	class Management(Authenticated):
		def has_permission(self, request, view) -> bool:
			_user: TokenUser = request.user
			return bool(super().has_permission(request, view) and _user.has_management_access)

	class IntraService(Authenticated):

		def has_permission(self, request, view) -> bool:
			_user: TokenUser = request.user
			return bool(
				super().has_permission(request, view) and _user.is_intra_service_requester)

	class ManagementOrIntraService(Authenticated):
		def has_permission(self, request, view) -> bool:
			_user: TokenUser = request.user
			return bool(
				super().has_permission(request, view) and
				(_user.has_management_access or _user.is_intra_service_requester)
			)



from msb.apis import (
	ApiView, api_details, SessionData, ApiResponse, RequestInfo, Permissions, require_permissions,
	RequestHeaders, ApiViewset, RestResponse, TokenUser

)
from msb.cipher import (Cipher)
from msb.dataclasses import (SearchParameter, SearchParameterRequest, Singleton)
from msb.db.models import (MsbModel, MsbModelMetaFields, MsbModelManager, model_fields)
from msb.exceptions import (ApiException, AppException, CrudApiException)
from msb.services import (ApiService, ApiServiceExceptions, CrudApiException)
from msb.validation import (require_inputs, DefaultRules, ValidationSchema, InputField, Validate,validation_schema_wrapper)

__all__ = [

	# core msb_apis imports
	"ApiView", "ApiViewset", "api_details", "ApiService", "require_permissions", "require_inputs", "Permissions",
	"TokenUser", "SessionData", "ApiResponse", "RequestInfo", "RequestHeaders", "RestResponse",

	# core msb_exceptions imports
	"ApiException", "CrudApiException", "AppException", "ApiServiceExceptions",

	# core msb_validation imports
	"DefaultRules", "ValidationSchema", "InputField", "Validate","validation_schema_wrapper",

	# core msb_db imports
	"MsbModel", "MsbModelMetaFields", "MsbModelManager", "model_fields",

	# core msb_dataclasses imports
	"SearchParameter", "SearchParameterRequest", "Singleton",

	# others
	"Cipher"
]



from .api_view import (
	ApiView, api_details, TokenUser, SessionData, ApiResponse, RequestInfo,
	RequestHeaders, Permissions, require_permissions
)
from .api_viewset import (ApiViewset, RestResponse)
from .constants import *

__all__ = [
	"ApiView", "ApiViewset", "api_details", "Permissions", "require_permissions",
	"TokenUser", "SessionData", "ApiResponse",
	"RequestInfo", "RequestHeaders", "RestResponse"
]
